/*
  Rodent, a UCI chess playing engine derived from Sungorus 1.4
  Copyright (C) 2009-2011 Pablo Vazquez (Sungorus author)
  Copyright (C) 2011-2014 Pawel Koziol

  Rodent is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published 
  by the Free Software Foundation, either version 3 of the License, 
  or (at your option) any later version.

  Rodent is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty 
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rodent.h"
#include "bitboard.h"
#include "eval.h"

int sEvaluator::EvalTrappedRook(sPos *p, int sd) 
{
	int score = 0;
	const U64 bbKTrapKs[2] = { SqBb(F1) | SqBb(G1), SqBb(F8) | SqBb(G8) };
	const U64 bbRTrapKs[2] = { SqBb(G1) | SqBb(H1) | SqBb(H2), SqBb(G8) | SqBb(H8) | SqBb(H7) };
	const U64 bbKTrapQs[2] = { SqBb(C1) | SqBb(B1), SqBb(C8) | SqBb(B8) };
	const U64 bbRTrapQs[2] = { SqBb(B1) | SqBb(A1) | SqBb(A2), SqBb(B8) | SqBb(A8) | SqBb(A7) };

  // rook blocked by uncastled king
  // TODO: test lower value
  if ( ( bbPc(p, sd, K) & bbKTrapKs[sd] ) && ( bbPc(p, sd, R) & bbRTrapKs[sd] ) ) score -= 50;
  if ( ( bbPc(p, sd, K) & bbKTrapQs[sd] ) && ( bbPc(p, sd, R) & bbRTrapQs[sd] ) ) score -= 50;

  return score;
}

int sEvaluator::EvalTrappedBishop(sPos *p, int sd) 
{
	int score = 0;
	int op = Opp(sd);
	const U64 bbTrapSq[2] = { SqBb(A7) | SqBb(B8) | SqBb(H7) | SqBb(G8) | SqBb(A6) | SqBb(H6) ,
		                      SqBb(A2) | SqBb(B1) | SqBb(H2) | SqBb(G1) | SqBb(A3) | SqBb(H3) };

  if ( bbPc(p, sd, B) & bbTrapSq[sd] ) {
     if ( ( bbPc(p, sd, B) & RelSqBb(A7,sd) ) && ( bbPc(p, op, P) & RelSqBb(B6,sd) ) ) score -= 150;
     if ( ( bbPc(p, sd, B) & RelSqBb(B8,sd) ) && ( bbPc(p, op, P) & RelSqBb(C7,sd) ) ) score -= 150;
     if ( ( bbPc(p, sd, B) & RelSqBb(H7,sd) ) && ( bbPc(p, op, P) & RelSqBb(G6,sd) ) ) score -= 150;
     if ( ( bbPc(p, sd, B) & RelSqBb(G8,sd) ) && ( bbPc(p, op, P) & RelSqBb(F7,sd) ) ) score -= 150;
     if ( ( bbPc(p, sd, B) & RelSqBb(A6,sd) ) && ( bbPc(p, op, P) & RelSqBb(B5,sd) ) ) score -= 50;
     if ( ( bbPc(p, sd, B) & RelSqBb(H6,sd) ) && ( bbPc(p, op, P) & RelSqBb(G5,sd) ) ) score -= 50;
  }

  return score;
}

int sEvaluator::EvalTrappedKnight(sPos *p, int sd) 
{
  int score = 0;
  int op = Opp(sd);
  const U64 bbTrapSq[2] = { SqBb(A7) | SqBb(A6) | SqBb(H7) | SqBb(H6),
	                        SqBb(A2) | SqBb(A3) | SqBb(H2) | SqBb(H3) };
  
   // trapped knight
   if (bbPc(p, sd, N) & bbTrapSq[sd]) {

      if ( bbPc(p, sd, N) & RelSqBb(A7,sd) ) {
         if ( bbPc(p, op, P) & RelSqBb(A6,sd) )  {
            score -= 75;
	        if ( bbPc(p, op, P) & RelSqBb(B7,sd) ) score -= 75;
	     }
      }

      if ( bbPc(p, sd, N) & RelSqBb(H7,sd) ) {
	     if ( bbPc(p, op, P) & RelSqBb(H6,sd) )  {
	  	    score -= 75;
	        if ( bbPc(p, op, P) & RelSqBb(G7,sd) ) score -= 75;
	     }
      }

      if ( bbPc(p, sd, N) & RelSqBb(A6,sd) ) {
	     if ( ( bbPc(p, op, P) & RelSqBb(A5,sd) ) 
		 && ( bbPc(p, op, P) & RelSqBb(B6,sd) ) )  score -= 75;	  
      }

	  if ( bbPc(p, sd, N) & RelSqBb(H6,sd) ) {
	     if ( ( bbPc(p, op, P) & RelSqBb(H5,sd) ) 
		 && ( bbPc(p, op, P) & RelSqBb(G6,sd) ) )  score -= 75;	  
      }
   }

  return score;
}
