/*
  Rodent, a UCI chess playing engine derived from Sungorus 1.4
  Copyright (C) 2009-2011 Pablo Vazquez (Sungorus author)
  Copyright (C) 2011-2014 Pawel Koziol

  Rodent is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published 
  by the Free Software Foundation, either version 3 of the License, 
  or (at your option) any later version.

  Rodent is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty 
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rodent.h"
#include "bitboard.h"
#include "data.h"

void sPos::UndoMove(int move, UNDO *u)
{
  int sd   = Opp(side);   // moving side
  int op   = side;
  int fsq  = Fsq(move);      // start square
  int tsq  = Tsq(move);      // target square
  int ftp  = Tp(pc[tsq]);    // moving piece
  int ttp  = u->ttp;         // captured piece

  U64 bbMove = SqBb(fsq) | SqBb(tsq); // optimization from Stockfish

  castleFlags = u->castleFlags;
  epSquare = u->epSquare;
  reversibleMoves = u->reversibleMoves;
  hashKey = u->hashKey;
  pawnKey = u->pawnKey;
  head--;

  pc[fsq] = Pc(sd, ftp);
  pc[tsq] = NO_PC;
  bbCl[sd] ^= bbMove;
  bbTp[ftp]  ^= bbMove;
  pstMg[sd] += Data.pstMg[sd][ftp][fsq] - Data.pstMg[sd][ftp][tsq]; 
  pstEg[sd] += Data.pstEg[sd][ftp][fsq] - Data.pstEg[sd][ftp][tsq]; 
  
  // on king move update king location data
  if (ftp == K) kingSquare[sd] = fsq;

  // capture
  if (ttp != NO_TP) {
    pc[tsq] = Pc(op, ttp);
    bbCl[op] ^= SqBb(tsq);
    bbTp[ttp]    ^= SqBb(tsq);
	pcCount[op] [ttp]++;
    pieceMat[op] += Data.matValue[ttp];
	phase        += Data.phaseValue[ttp];
    pstMg[op]    += Data.pstMg[op][ttp][tsq];
	pstEg[op]    += Data.pstEg[op][ttp][tsq];
  }

  switch (MoveType(move)) {

  case NORMAL:
    break;

  case CASTLE:
    if (tsq > fsq) {
      fsq += 3;
      tsq -= 1;
    } else {
      fsq -= 4;
      tsq += 1;
    }
    pc[tsq] = NO_PC;
    pc[fsq] = Pc(sd, R);
    bbCl[sd] ^= SqBb(fsq) | SqBb(tsq);
    bbTp[R] ^= SqBb(fsq) | SqBb(tsq);
    pstMg[sd] += Data.pstMg[sd][R][fsq] - Data.pstMg[sd][R][tsq];
	pstEg[sd] += Data.pstEg[sd][R][fsq] - Data.pstEg[sd][R][tsq];
    break;

  case EP_CAP:
    tsq ^= 8;
    pc[tsq] = Pc(op, P);
    bbCl[op] ^= SqBb(tsq);
    bbTp[P] ^= SqBb(tsq);
	pcCount[op] [P]++;
	phase            += Data.phaseValue[P];
    pstMg[op] += Data.pstMg[op][P][tsq];
	pstEg[op] += Data.pstEg[op][P][tsq];
    break;

  case EP_SET:
    break;

  case N_PROM: case B_PROM: case R_PROM: case Q_PROM:
    pc[fsq] = Pc(sd, P);
    bbTp[P]   ^= SqBb(fsq);
    bbTp[ftp] ^= SqBb(fsq);
	pcCount[sd][P]++;
	pcCount[sd][ftp]--;
    pieceMat[sd] -= Data.matValue[ftp];
	phase        += Data.phaseValue[P]     - Data.phaseValue[ftp];
    pstMg[sd]    += Data.pstMg[sd][P][fsq] - Data.pstMg[sd][ftp][fsq];
	pstEg[sd]    += Data.pstEg[sd][P][fsq] - Data.pstEg[sd][ftp][fsq];
    break;
  }
  side ^= 1;
}

void sPos::UndoNull(UNDO *u)
{
  epSquare = u->epSquare;
  hashKey = u->hashKey;
  pawnKey = u->pawnKey;
  head--;
  reversibleMoves--;
  side ^= 1;
}
